import java.util.ArrayList;

/**
 *  This class is the main class of the "World of Zuul" application. 
 *  "World of Zuul" is a very simple, text based adventure game.  Users 
 *  can walk around some scenery. That's all. It should really be extended 
 *  to make it more interesting!
 * 
 *  To play this game, create an instance of this class and call the "play"
 *  method.
 * 
 *  This main class creates and initialises all the others: it creates all
 *  rooms, creates the parser and starts the game.  It also evaluates and
 *  executes the commands that the parser returns.
 * 
 * pld notes:
 * 
 * In this version, GO is handled by Room.response(), like other command words.
 * 
 * In this version, Player info is part of the Player class, and is passed
 * as parameter to all Room methods.
 * 
 */

public class Game 
{
    private Parser parser;
    private Player player;
    private Room outside;

//  private static Game theGame = null;
    
   /**
    * Standard constructor
    */
    public Game() {
        parser = new Parser();
        outside = RoomMaker.createRooms();
        player = new Player(outside);
        player.getInventory().add(new Light("flashlight", 4));
        player.getInventory().add(new Item("cookie", 2));  
    }
   
 
    /**
     *  Main play routine.  Loops until end of play.
     */
    public void play() 
    {            
        printWelcome();

        // Enter the main command loop.  Here we repeatedly read commands and
        // execute them until the game is over.
                
        boolean finished = false;
        while (! finished) {
            Command command = parser.getCommand();
            finished = processCommand(command);
            if (player.hasDied()) finished = true;
        }
        System.out.println("Thank you for playing.  Good bye.");
    }

    /**
     * Print out the opening message for the player.
     */
    private void printWelcome()
    {
        System.out.println();
        System.out.println("Welcome to the World of Zuul!");
        System.out.println("World of Zuul is a new, incredibly boring adventure game.");
        System.out.println("Type 'help' if you need help.");
        System.out.println();
        Room currentRoom = player.getCurrentRoom();
        System.out.println(currentRoom.getLongDescription(player));
    }

    /**
     * Given a command, process (that is: execute) the command.
     * If this command ends the game, true is returned, otherwise false is
     * returned.
     */
    private boolean processCommand(Command command) 
    {
        boolean wantToQuit = false;

        if(command.isUnknown()) {
            System.out.println("I don't know what you mean...");
            return false;
        }

        String commandWord = command.getCommandWord();
        
        if (commandWord.equals("quit")) {      
            return true;        
        } else if (commandWord.equals("help")) {
            printHelp();
        } else if (commandWord.equals("inventory")) {
            printInventory(player);
        } else if (commandWord.equals("look")) {
            System.out.println(player.getCurrentRoom().getLongDescription(player));
        } else {
            Room r = player.getCurrentRoom().respond(command, player);
            if (r != null) player.setCurrentRoom(r);
        }
              
        return wantToQuit;      // always false
    }


    /**
     * Print out some help information.
     * Here we print some stupid, cryptic message and a list of the 
     * command words.
     */
    private void printHelp() 
    {
        System.out.println("You are lost. You are alone. You wander");
        System.out.println("around at the university.");
        System.out.println();
        System.out.println("Your command words are:");
        parser.showCommands();
    }

    /** 
     * "Quit" was entered. Check the rest of the command to see
     * whether we really quit the game. Return true, if this command
     * quits the game, false otherwise.
     */
    private boolean quit(Command command) 
    {
        if(command.hasSecondWord()) {
            System.out.println("Quit what?");
            return false;
        }
        else
            return true;  // signal that we want to quit
    }
    
    /**
     * printInventory prints the contents of "inventory"
     */
    private void printInventory(Player p) {
        ArrayList<Item> inventory = p.getInventory();
        if (inventory.size() == 0) {
            System.out.println("you are not carrying anything");
        } else {
            System.out.print("You have the following:");
            for (int n = 0; n < inventory.size(); n++) {
                Item item = (Item) inventory.get(n);
                System.out.print(" " + item.getDesc());
            }
            System.out.println();
            System.out.println("Total weight: " + totalWeight(inventory));
        }
    }
    
    /**
     * findByName: given a string and an ArrayList of Items, 
     * find the Item with the matching name, or else return null
     */
    static public Item findByName(String s, ArrayList<Item> L) {
        int n=0;
        while (n < L.size()) {
            Item i =  L.get(n);
            if (s.equals(i.getDesc())) 
                return i;
            n++;
        }
        return null;    // not found above
    }
   
   /**
     * totalWeight: given an ArrayList of Items, returns total of weights
     */
    public static int totalWeight(ArrayList<Item> L) {
        int n=0;
        int sum = 0;
        while (n < L.size()) {
            Item i =  L.get(n);
            sum += i.getWeight();
            n++;
        }
        return sum;    // not found above
    }
    
    /* 
    public static void main(String [] args) {
        initGame();
        Game g = getGame();
        g.play();
    }
    
    /**
     * initGame(): like ctor, except checks that there isn't already one
     *
    public static void initGame() {
        if (theGame != null) {
            theGame = new Game();
        }
    }
    
    /** 
     * for access to the game from outside.
     * Problem: what if there are multiple games?
     
    public static Game getGame() {
        return theGame;
    }
    /* */
    /**
     * Alternative constructor, not used
     *
    public void Game() 
    {
        Game theGame = null;
        if (theGame != null) { // second game
            //inventory = player.getInventory();
            //player.setCurrentRoom(outside);
            parser = theGame.parser;
        } else {// new game
            
            parser = new Parser();
            outside = RoomMaker.createRooms();
            player = new Player(outside);
            player.getInventory().add(new Light("flashlight", 4));
            player.getInventory().add(new Item("cookie", 2));  
            theGame = this;
        }
    }
     
    /* */
}
